#ifndef BIOMETRIC_COMM_H
#define BIOMETRIC_COMM_H

#include <stdbool.h>
#include <limits.h>
#include <glib.h>
#include <libusb.h>
#include <pthread.h>

#include "biometric_stroge.h"

#define MAX_PATH_LEN 1024

#define BIO_OPS_TIMEOUT 30

typedef enum {
	BioT_FingerPrint,		/** 指纹 **/
	BioT_FingerVein,		/** 指静脉 **/
	BioT_Iris,				/** 虹膜 **/
	BioT_Face,				/** 人脸 **/
}BioType;

typedef enum {
	StoT_Device,			/** 存储在硬件中 **/
	StoT_OS,				/** 存储在操作系统中 **/
}StoType;

typedef enum {
	EigT_Data,				/** 原始数据 **/
	EigT_Eigenvalue,		/** 特征值 **/
	EigT_Eigenvector,		/** 特征向量 **/
}EigType;

typedef enum {
	VerT_Hardware,			/** 硬件验证 **/
	VerT_Software,			/** 软件验证 **/
	VerT_Mix,				/** 混合验证 **/
	VerT_Other
}VerType;

typedef enum {
	IdT_Hardware,			/** 硬件识别 **/
	IdT_Software,			/** 软件识别 **/
	IdT_Mix,				/** 混合识别 **/
	IdT_Other
}IdType;

typedef enum {
	BusT_Serial,			/** 串口 **/
	BusT_USB,				/** USB接口 **/
	BusT_PCIE,				/** PCIE接口 **/
}BusType;

typedef enum {
	ACTION_START,			/** 开始操作 **/
	ACTION_STOP,			/** 中断操作 **/
}OpsActions;

struct Serial_Info {
	int fd;
	char path[MAX_PATH_LEN];
	unsigned long driver_data;
};

struct USB_INFO {
	int vendor_id;
	int product_id;
	int class_id;
	libusb_hotplug_callback_handle callback_handle[2];
	unsigned long driver_data;
};

struct PCI_INFO {
	int vendor_id;
	int product_id;
	unsigned long driver_date;
};

struct BioInfo{
	BioType biotype;		/** 生物特征类型 **/
	StoType stotype;		/** 存储类型 **/
	EigType eigtype;		/** 特征类型 **/
	VerType vertype;		/** 验证类型 **/
	IdType idtype;			/** 识别类型 **/
	BusType bustype;		/** 总线类型 **/
	void *biodata;			/** 具体各特征私有信息 **/
};

typedef enum {
	DEVS_COMM_IDLE = 0,			/** 空闲状态 **/
	DEVS_COMM_DOING,			/** 设备正在处理中 **/
	DEVS_COMM_STOP_BY_USER,		/** 设备操作被终止 **/
	DEVS_COMM_DISABLE,			/** 设备被禁用 **/
	DEVS_COMM_MAX,

	DEVS_OPEN_DOING = 101,		/** 正在打开设备 **/
	DEVS_OPEN_STOP_BY_USER,		/** 打开操作被终止 **/
	DEVS_OPEN_MAX,

	DEVS_ENROLL_DOING =201,		/** 正在录入 **/
	DEVS_ENROLL_STOP_BY_USER,		/** 打开操作被终止 **/
	DEVS_ENROLL_MAX,

	DEVS_VERIFY_DOING = 301,	/** 正在认证 **/
	DEVS_VERIFY_STOP_BY_USER,		/** 打开操作被终止 **/
	DEVS_VERIFY_MAX,

	DEVS_IDENTIFY_DOING = 401,	/** 正在识别指定特征 **/
	DEVS_IDENTIFY_STOP_BY_USER,		/** 打开操作被终止 **/
	DEVS_IDENTIFY_MAX,

	DEVS_CAPTURE_DOING = 501,	/** 正在捕获信息 **/
	DEVS_CAPTURE_STOP_BY_USER,		/** 打开操作被终止 **/
	DEVS_CAPTURE_MAX,

	DEVS_SEARCH_DOING = 601,		/** 正在搜索指定特征 **/
	DEVS_SEARCH_STOP_BY_USER,		/** 打开操作被终止 **/
	DEVS_SEARCH_MAX,

	DEVS_CLEAN_DOING = 701,			/** 正在清理特征数据 **/
	DEVS_CLEAN_STOP_BY_USER,		/** 打开操作被终止 **/
	DEVS_CLEAN_MAX,

	DEVS_GET_FLIST_DOING = 801,		/** 正在获取特征列表 **/
	DEVS_GET_FLIST_STOP_BY_USER,		/** 打开操作被终止 **/
	DEVS_GET_FLIST_MAX,

	DEVS_CLOSE_DOING = 901,			/** 正在关闭设备 **/
	DEVS_CLOSE_STOP_BY_USER,		/** 打开操作被终止 **/
	DEVS_CLOSE_MAX,
}DevStatus;

typedef enum {
	OPS_COMM_SUCCESS = 0,		/** 空闲状态 **/
	OPS_COMM_FAIL,				/** 操作失败 **/
	OPS_COMM_ERROR,
	OPS_COMM_STOP_BY_USER,		/** 用户中断 **/
	OPS_COMM_TIMEOUT,			/** 操作超时 **/
	OPS_COMM_OUT_OF_MEM,
	OPS_COMM_MAX,

	OPS_OPEN_SUCCESS = 100,			/** 打开设备完成 **/
	OPS_OPEN_FAIL,					/** 打开设备失败 **/
	OPS_OPEN_ERROR,
	OPS_OPEN_MAX,

	OPS_ENROLL_SUCCESS = 200,		/** 录入信息成功 **/
	OPS_ENROLL_FAIL,				/** 录入失败 **/
	OPS_ENROLL_ERROR,
	OPS_ENROLL_STOP_BY_USER,		/** 录入被用户中断 **/
	OPS_ENROLL_TIMEOUT,				/** 操作超时 **/
	OPS_ENROLL_MAX,

	OPS_VERIFY_MATCH = 300,			/** 认证匹配 **/
	OPS_VERIFY_NO_MATCH,			/** 认证不匹配 **/
	OPS_VERIFY_ERROR,
	OPS_VERIFY_STOP_BY_USER,		/** 认证被用户中断 **/
	OPS_VERIFY_TIMEOUT,				/** 操作超时 **/
	OPS_VERIFY_MAX,

	OPS_IDENTIFY_MATCH = 400,		/** 识别到指定特征 **/
	OPS_IDENTIFY_NO_MATCH,			/** 未识别出指定特征 **/
	OPS_IDENTIFY_ERROR,				/** 识别过程中遇到错误 **/
	OPS_IDENTIFY_STOP_BY_USER,		/** 识别被用户中断 **/
	OPS_IDENTIFY_TIMEOUT,			/** 操作超时 **/
	OPS_IDENTIFY_MAX,

	OPS_CAPTURE_SUCCESS = 500,		/** 捕获成功 **/
	OPS_CAPTURE_FAIL,				/** 捕获失败 **/
	OPS_CAPTURE_ERROR,				/** 捕获过程中遇到错误 **/
	OPS_CAPTURE_STOP_BY_USER,		/** 捕获被用户中断 **/
	OPS_CAPTURE_TIMEOUT,			/** 操作超时 **/
	OPS_CAPTURE_MAX,

	OPS_SEARCH_MATCH = 600,			/** 搜索到指定特征 **/
	OPS_SEARCH_NO_MATCH,			/** 未搜索到指定特征 **/
	OPS_SEARCH_ERROR,				/** 搜索过程中遇到错误 **/
	OPS_SEARCH_STOP_BY_USER,		/** 搜索被用户中断 **/
	OPS_SEARCH_TIMEOUT,				/** 操作超时 **/
	OPS_SEARCH_MAX,

	OPS_CLEAN_SUCCESS = 700,		/** 清理特征成功 **/
	OPS_CLEAN_FAIL,					/** 清理失败 **/
	OPS_CLEAN_ERROR,				/** 清理过程中遇到错误 **/
	OPS_CLEAN_STOP_BY_USER,			/** 清理被用户中断 **/
	OPS_CLEAN_TIMEOUT,				/** 操作超时 **/
	OPS_CLEAN_MAX,

	OPS_GET_FLIST_SUCCESS = 800,		/** 获取特征列表完成 **/
	OPS_GET_FLIST_FAIL,					/** 获取特征列表失败 **/
	OPS_GET_FLIST_ERROR,
	OPS_GET_FLIST_STOP_BY_USER,			/** 获取特征列表被用户中断 **/
	OPS_GET_FLIST_TIMEOUT,				/** 获取特征列表超时 **/
	OPS_GET_FLIST_MAX,

	OPS_CLOSE_SUCCESS = 900,		/** 关闭设备完成 **/
	OPS_CLOSE_FAIL,					/** 关闭设备失败 **/
	OPS_CLOSE_ERROR,
	OPS_CLOSE_MAX,
}OpsStatus;

typedef enum {
	NOTIFY_COMM_IDLE = 0,		/** 空闲状态 **/
	NOTIFY_COMM_SUCCESS = 0,
	NOTIFY_COMM_FAIL,				/** 操作失败 **/
	NOTIFY_COMM_ERROR,
	NOTIFY_COMM_STOP_BY_USER,		/** 用户中断 **/
	NOTIFY_COMM_TIMEOUT,			/** 操作超时 **/
	NOTIFY_COMM_DISABLE,
	NOTIFY_COMM_OUT_OF_MEM,
	NOTIFY_COMM_MAX,

	NOTIFY_OPEN_SUCCESS = 100,			/** 打开设备完成 **/
	NOTIFY_OPEN_FAIL,					/** 打开设备失败 **/
	NOTIFY_OPEN_ERROR,
	NOTIFY_OPEN_MAX,

	NOTIFY_ENROLL_SUCCESS = 200,		/** 录入信息成功 **/
	NOTIFY_ENROLL_FAIL,				/** 录入失败 **/
	NOTIFY_ENROLL_ERROR,
	NOTIFY_ENROLL_STOP_BY_USER,		/** 录入被用户中断 **/
	NOTIFY_ENROLL_TIMEOUT,				/** 操作超时 **/
	NOTIFY_ENROLL_MAX,

	NOTIFY_VERIFY_MATCH = 300,			/** 认证匹配 **/
	NOTIFY_VERIFY_NO_MATCH,			/** 认证不匹配 **/
	NOTIFY_VERIFY_ERROR,
	NOTIFY_VERIFY_STOP_BY_USER,		/** 认证被用户中断 **/
	NOTIFY_VERIFY_TIMEOUT,				/** 操作超时 **/
	NOTIFY_VERIFY_MAX,

	NOTIFY_IDENTIFY_MATCH = 400,		/** 识别到指定特征 **/
	NOTIFY_IDENTIFY_NO_MATCH,			/** 未识别出指定特征 **/
	NOTIFY_IDENTIFY_ERROR,				/** 识别过程中遇到错误 **/
	NOTIFY_IDENTIFY_STOP_BY_USER,		/** 识别被用户中断 **/
	NOTIFY_IDENTIFY_TIMEOUT,			/** 操作超时 **/
	NOTIFY_IDENTIFY_MAX,

	NOTIFY_CAPTURE_SUCCESS = 500,		/** 捕获成功 **/
	NOTIFY_CAPTURE_FAIL,				/** 捕获失败 **/
	NOTIFY_CAPTURE_ERROR,				/** 捕获过程中遇到错误 **/
	NOTIFY_CAPTURE_STOP_BY_USER,		/** 捕获被用户中断 **/
	NOTIFY_CAPTURE_TIMEOUT,			/** 操作超时 **/
	NOTIFY_CAPTURE_MAX,

	NOTIFY_SEARCH_MATCH = 600,			/** 搜索到指定特征 **/
	NOTIFY_SEARCH_NO_MATCH,			/** 未搜索到指定特征 **/
	NOTIFY_SEARCH_ERROR,				/** 搜索过程中遇到错误 **/
	NOTIFY_SEARCH_STOP_BY_USER,		/** 搜索被用户中断 **/
	NOTIFY_SEARCH_TIMEOUT,				/** 操作超时 **/
	NOTIFY_SEARCH_MAX,

	NOTIFY_CLEAN_SUCCESS = 700,		/** 清理特征成功 **/
	NOTIFY_CLEAN_FAIL,					/** 清理失败 **/
	NOTIFY_CLEAN_ERROR,				/** 清理过程中遇到错误 **/
	NOTIFY_CLEAN_STOP_BY_USER,			/** 清理被用户中断 **/
	NOTIFY_CLEAN_TIMEOUT,				/** 操作超时 **/
	NOTIFY_CLEAN_MAX,

	NOTIFY_GET_FLIST_SUCCESS = 800,		/** 获取特征列表完成 **/
	NOTIFY_GET_FLIST_FAIL,					/** 获取特征列表失败 **/
	NOTIFY_GET_FLIST_ERROR,
	NOTIFY_GET_FLIST_STOP_BY_USER,			/** 获取特征列表被用户中断 **/
	NOTIFY_GET_FLIST_TIMEOUT,				/** 获取特征列表超时 **/
	NOTIFY_GET_FLIST_MAX,

	NOTIFY_CLOSE_SUCCESS = 900,		/** 关闭设备完成 **/
	NOTIFY_CLOSE_FAIL,					/** 关闭设备失败 **/
	NOTIFY_CLOSE_ERROR,
	NOTIFY_CLOSE_MAX,
}NotifyStatus;

typedef struct bio_dev_t bio_dev;
struct bio_dev_t {
	// 设备、驱动基本信息
	int driver_id;	/** 驱动ID **/
	char *device_name;/** 设备名 **/
	char *full_name;	/** 设备全称 **/
	struct BioInfo bioinfo;	/** 设备的相关类型 **/

	// 总线信息
	struct Serial_Info	serial_info;
	struct USB_INFO		usb_info;
	struct PCI_INFO		pci_info;

	// 设备、驱动私有信息
	void *dev_priv;			/** 设备驱动私有结构体 **/
	void *bio_priv;			/** 各生物信息私有结构体 **/

	// 边界值
	int max_user;
	int max_sample;
	int sample_times;

	// 操作集合
	int (*ops_configure)(bio_dev *dev, GKeyFile * conf);
	int (*ops_discover)(bio_dev *dev);
	int (*ops_open)(bio_dev *dev);
	int (*ops_enroll)(bio_dev *dev, OpsActions action, int uid, int idx,
					  char * bio_idx_name);
	int (*ops_verify)(bio_dev *dev, OpsActions action, int uid, int idx);
	int (*ops_identify)(bio_dev *dev, OpsActions action, int uid,
						int idx_start, int idx_end);
	char * (*ops_capture)(bio_dev *dev, OpsActions action);
	feature_info * (*ops_search)(bio_dev *dev, OpsActions action, int uid,
					  int idx_start, int idx_end);
	int (*ops_clean)(bio_dev *dev, OpsActions action, int uid,
					 int idx_start, int idx_end);
	feature_info * (*ops_get_feature_list)(bio_dev *dev, OpsActions action,
										   int uid, int idx_start, int idx_end);
	void (*ops_attach)(bio_dev *dev);
	void (*ops_detach)(bio_dev *dev);
	int (*ops_stop_by_user)(bio_dev *dev, int waiting_ms);
	void (*ops_close)(bio_dev *dev);
	void (*ops_free)(bio_dev *dev);

	const char * (*ops_get_dev_status_mesg)(bio_dev *dev);
	const char * (*ops_get_ops_status_mesg)(bio_dev *dev);
	const char * (*ops_get_notify_status_mesg)(bio_dev *dev);

	volatile DevStatus dev_status;	/** 设备状态 **/
	volatile OpsStatus ops_status;	/** 操作结果 **/
	volatile NotifyStatus notify_status;	/** 给用户的提示 **/

	void * plugin_handle;
	char * capture_base64;
	int enable;
};

void bio_set_ops_timeout_ms(int t);
int bio_get_ops_timeout_ms();

GList * bio_drv_list;
GList * bio_dev_list;

void bio_free_driver(GList * );
void bio_free_device(GList * dev_list);

/* 提供的外部函数 */
feature_info * bio_ops_get_feature_list(bio_dev *dev, int uid,
										int idx_start, int idx_end);
int bio_ops_enroll(bio_dev *dev, int uid, int idx, char * idx_name);
int bio_ops_verify(bio_dev *dev, int uid, int idx);
int bio_ops_identify(bio_dev *dev, int uid, int idx_start, int idx_end);
char * bio_ops_capture(bio_dev *dev);
feature_info * bio_ops_search(bio_dev *dev, int uid, int idx_start, int idx_end);
int bio_ops_clean(bio_dev *dev, int uid, int idx_start, int idx_end);
int bio_ops_stop_ops(bio_dev *dev, int waiting_sec);

int bio_init(void);
int bio_dev_is_enable(bio_dev * dev, GKeyFile * conf);
int bio_dev_set_serial_path(bio_dev * dev, GKeyFile * conf);
bool bio_is_stop_by_user(bio_dev * dev);
GList * bio_driver_list_init(void);
GList * bio_device_list_init(GList *);

int bio_base64_encode(unsigned char * bin_data, char * base64, int bin_len );
int bio_base64_decode(char * base64, unsigned char * bin_data);

int bio_get_dev_status(bio_dev * dev);
void bio_set_dev_status(bio_dev * dev, int status);
//void bio_set_priv_dev_status(bio_dev * dev, int status);

int bio_get_ops_status(bio_dev * dev);
void bio_set_ops_status(bio_dev * dev, int status);
void bio_set_ops_abs_status(bio_dev * dev, int status);

int bio_get_notify_status(bio_dev * dev);
void bio_set_notify_status(bio_dev * dev, int status);
void bio_set_notify_abs_status(bio_dev * dev, int status);

const char * bio_get_dev_status_mesg(bio_dev *dev);
const char * bio_get_ops_status_mesg(bio_dev *dev);
const char * bio_get_notify_status_mesg(bio_dev *dev);

GList * bio_get_drv_list(void);
GList * bio_get_dev_list(void);

void bio_free_drv_list(void);
void bio_free_dev_list(void);

int bio_close(void);

char * bio_new_string(char * old_str);

#endif // BIOMETRIC_COMM_H

